;##################################################################
;
;   TesbWan for TI-86 (Ball movement)
;
;   Programmed by Patrick Davidson (pad@calc.org)
;        
;   This program is in the public domain.  There is no warranty.
;
;   This file was last updated August 22, 2002.
;
;##################################################################

handle_ball:
        ld      de,(prev_coords)
        call    draw_ball

        ld      de,(ball_x)
        ld      a,(ball_y+1)
        ld      e,a
        ld      (prev_coords),de
        call    draw_ball

        ld      a,(ball_speed)
        ld      b,a
lb:     push    bc
        call    move_ball
        pop     bc
        djnz    lb

        ld      de,(ball_x)
        ld      a,(ball_y+1)
        ld      e,a

draw_ball:
        ld      hl,ball_image
        jp      drw_spr

;############## Ball movement

move_ball:
        ld      a,(ball_type)
        or      a
        ret     nz

;############## X movement

        ld      a,(ball_angle)          ; X movement
        ld      hl,angles_data
        call    ADD_HL_A
        ld      a,(hl)                  ; A = abs XV
        ld      d,0
        ld      e,a                     ; DE = abs XV
        ld      hl,(ball_x)             ; HL = x
        ld      (old_x),hl

        ld      a,(ball_xdir)
        or      a
        jr      nz,move_ball_right

;############## Movement to left

        sbc     hl,de                   ; move the ball
        ld      (ball_x),hl

        ld      a,$ff                   ; check if hit left side of screen
        cp      h
        jr      nz,no_left_screen_hit
        ld      hl,0
        ld      (ball_x),hl
        ld      a,1
        ld      (ball_xdir),a
        jr      move_ball_y

no_left_screen_hit:                     ; check for hitting brick
        ld      d,h
        ld      a,(ball_y+1)
        inc     a
        call    check_brick_hit
        jr      z,move_ball_y
        ld      hl,(old_x)
        ld      (ball_x),hl
        ld      a,1
        ld      (ball_xdir),a
        jr      move_ball_y

;############## Movement to right

move_ball_right:
        add     hl,de                   ; move the ball
        ld      (ball_x),hl

        ld      a,(MAP_WIDTH*8)-2
        cp      h                       ; check if hit right side of screen
        jr      nz,no_right_screen_hit
        ld      hl,(((MAP_WIDTH*8)-2)<<8)+$FF
        ld      (ball_x),hl
        xor     a
        ld      (ball_xdir),a

no_right_screen_hit:                    ; check for hitting brick
        ld      d,h
        inc     d
        inc     d
        ld      a,(ball_y+1)
        inc     a
        call    check_brick_hit
        jr      z,move_ball_y
        ld      hl,(old_x)
        ld      (ball_x),hl
        xor     a
        ld      (ball_xdir),a
        jr      move_ball_y

;############## Y movement

move_ball_y:
        ld      a,(ball_angle)          ; Y movement
        cpl
        add     a,9
        ld      hl,angles_data
        call    ADD_HL_A
        ld      a,(hl)                  ; A = abs YV
        ld      d,0
        ld      e,a                     ; DE = abs YV
        ld      hl,(ball_y)             ; HL = y
        ld      (old_y),hl

        ld      a,(ball_ydir)
        or      a
        jr      nz,move_ball_down

;############## Movement up

        sbc     hl,de
        ld      (ball_y),hl

        bit     7,h                     ; check if hit top of screen
        jr      z,no_top_hit       
        ld      hl,0
        ld      (ball_y),hl
        ld      a,1
        ld      (ball_ydir),a
        ret

no_top_hit:
        ld      a,(ball_x+1)
        inc     a
        ld      d,a
        ld      a,h
        call    check_brick_hit
        ret     z
        ld      hl,(old_y)
        ld      (ball_y),hl
        ld      a,1
        ld      (ball_ydir),a
        ret

;############## Movement down

move_ball_down:
        add     hl,de
        ld      (ball_y),hl

        ld      a,62                    ; test for below bottom
        cp      h
        jr      z,ball_gone

        ld      a,57                    ; test for paddle-hitting Y position
        cp      h
        jr      nc,no_paddle_hit

        ld      a,(paddle_x)            ; compare X coordinate with paddle
        ld      e,a
        ld      a,(ball_x+1)
        add     a,2
        sub     e
        jr      c,no_paddle_hit

        ld      hl,long_paddle
        bit     0,(hl)
        jr      z,no_long_paddle
        cp      27
        jr      nc,no_paddle_hit
        add     a,a
        add     a,a
        call    DIV_A_6
        ld      a,c
no_long_paddle:

        cp      18
        jr      nc,no_paddle_hit
        cp      9
        jr      c,bounce_left

        ld      hl,$0001                ; bounce off right half of paddle          
        ld      (ball_xdir),hl
        sub     9
        ld      (ball_angle),a
        ret

bounce_left:
        ld      hl,$0000                ; bounce off left half of paddle           
        ld      (ball_xdir),hl
        cpl           
        add     a,9
        ld      (ball_angle),a
        ret

no_paddle_hit:
        ld      a,(ball_x+1)            ; check for hitting brick
        inc     a
        ld      d,a
        ld      a,(ball_y+1)
        add     a,2
        call    check_brick_hit
        ret     z
        ld      hl,(old_y)
        ld      (ball_y),hl
        xor     a
        ld      (ball_ydir),a
        ret

ball_gone:
        ld      a,2
        ld      (ball_type),a
        ret

;############## Angle sines (starting at 25 degrees, ending at 65)

angles_data:
        .db     32,40,44,52,56,60,64,68,72

;############## Ball image

ball_image:
        .db     3
        .db     %11100000
        .db     %11100000
        .db     %11100000
        .db     %11100000
        .db     %11100000
        .db     %11100000
