;##################################################################
;
;   TesBwan - Sprite drawing routine for background
;
;   Programmed by Patrick Davidson (pad@calc.org)
;        
;   This program is in the public domain.  There is no warranty.
;
;   This file was last updated August 22, 2002.
;
;##################################################################     

drw_block:
        ld      a,(screen_x)
        cpl
        add     a,d                     ; A = X coordinate
        add     a,9
        bit     7,a                     ; X >= 128 -> completely offscreen
        ret     nz
        cp      120
        ret     nc

        ld      d,a

        and     7
        ld      b,a
        add     a,a
        add     a,b
        ld      (bjumpintable+1),a       ; Save selected shift amount 

        push    hl

        ld      b,0
        ld      hl,back_buffer          ; always drawing into back-buffer
        ld      a,e

        add     a,a                     ; A = Y * 2
        add     a,a
        rl      b                       ; BA = Y * 4
        add     a,a                    
        rl      b                       ; BA = Y * 8
        add     a,a
        rl      b                       ; BA = Y * 16
        ld      c,d             
        srl     c
        srl     c
        srl     c                       ; C = X / 8
        or      c
        ld      c,a
        add     hl,bc                   ; HL = Screen address

        ex      de,hl                   ; DE = Screen address

        pop     hl                      ; HL -> image

        ex      de,hl                   ; HL -> screen, DE -> image

        ld      b,6
                
        ld      c,0
        scf
        ccf

        push    hl
        push    bc
        call    bjumpintable
        pop     bc
        pop     hl
        inc     h
        inc     h
        inc     h
        inc     h

bjumpintable:
        jr      btable

btable:
        jp      broutine0
        jp      broutine1
        jp      broutine2
        jp      broutine3
        jp      broutine4
        jp      broutine5
        jp      broutine6
        jp      broutine7

;############## Non-clipped drawing

broutine0:
        ld      c,16            ;7
broutine0_:
        ld      a,(de)          ;7
        inc     de              ;6

        xor      (hl)            ;7
        ld      (hl),a          ;7

        ld      a,c             ;4
        add     a,l             ;4
        ld      l,a             ;4
        jp      nc,bdone0        ;10
        inc     h               ;4
bdone0:  djnz    broutine0_
        ret

broutine7:
        inc     hl
broutine7_:
        ld      a,(de)          ;7
        inc     de              ;6

        add     a,a             ;4
        rl      c               ;8
        xor      (hl)            ;7
        ld      (hl),a          ;7
        dec     hl              ;6
        ld      a,(hl)          ;7
        xor      c               ;4
        ld      (hl),a          ;7

        ld      a,b             ;4
        ld      bc,17           ;10
        add     hl,bc           ;11
        ld      c,b             ;4
        ld      b,a             ;4
        djnz    broutine7_
        ret

broutine1:
        ld      a,(de)          ;7
        inc     de              ;6

        rra                     ;4
        rr      c               ;8
        xor      (hl)            ;7
        ld      (hl),a          ;7
        inc     hl              ;6
        ld      a,(hl)          ;7
        xor      c               ;4
        ld      (hl),a          ;7

        ld      a,b             ;4
        ld      bc,15           ;10
        add     hl,bc           ;11
        ld      c,b             ;4
        ld      b,a             ;4
        djnz    broutine1
        ret

broutine2:
        ld      a,(de)          ;7
        inc     de              ;6

        rrca                    ;4
        rrca                    ;4
        ld      c,a             ;4
        and     $3F             ;7
        xor      (hl)            ;7
        ld      (hl),a          ;7
        ld      a,c             ;4
        and     $C0             ;7
        inc     hl              ;6
        xor      (hl)            ;7
        ld      (hl),a          ;7

        ld      a,15            ;7
        add     a,l             ;4
        ld      l,a             ;4
        jp      nc,bdone2        ;10
        inc     h               ;4
bdone2:  djnz    broutine2
        ret

broutine3:
        ld      a,(de)          ;7
        inc     de              ;6

        rrca                    ;4
        rrca                    ;4
        rrca                    ;4
        ld      c,a             ;4
        and     $1F             ;7
        xor      (hl)            ;7
        ld      (hl),a          ;7
        ld      a,c             ;4
        and     $E0             ;7
        inc     hl              ;6
        xor      (hl)            ;7
        ld      (hl),a          ;7

        ld      a,15            ;7
        add     a,l             ;4
        ld      l,a             ;4
        jp      nc,bdone3        ;10
        inc     h               ;4
bdone3:  djnz    broutine3
        ret

broutine4:
        ld      a,(de)          ;7
        inc     de              ;6

        rrca                    ;4
        rrca                    ;4
        rrca                    ;4
        rrca                    ;4
        ld      c,a             ;4
        and     $0F             ;7
        xor      (hl)            ;7
        ld      (hl),a          ;7
        ld      a,c             ;4
        and     $F0             ;7
        inc     hl              ;6
        xor      (hl)            ;7
        ld      (hl),a          ;7

        ld      a,15            ;7
        add     a,l             ;4
        ld      l,a             ;4
        jp      nc,bdone4        ;10
        inc     h               ;4
bdone4:  djnz    broutine4        ;13
        ret

broutine6:
        ld      a,(de)          ;7
        inc     de              ;6

        rlca                    ;4
        rlca                    ;4
        ld      c,a             ;4
        and     $03             ;7
        xor      (hl)            ;7
        ld      (hl),a          ;7
        ld      a,c             ;4
        and     $FC             ;7
        inc     hl              ;6
        xor      (hl)            ;7
        ld      (hl),a          ;7

        ld      a,15            ;7
        add     a,l             ;4
        ld      l,a             ;4
        jp      nc,bdone6        ;10
        inc     h               ;4
bdone6:  djnz    broutine6        ;13
        ret
                               
broutine5:
        ld      a,(de)          ;7
        inc     de              ;6

        rlca                    ;4
        rlca                    ;4
        rlca                    ;4
        ld      c,a             ;4
        and     $07             ;7
        xor      (hl)            ;7
        ld      (hl),a          ;7
        ld      a,c             ;4
        and     $F8             ;7
        inc     hl              ;6
        xor      (hl)            ;7
        ld      (hl),a          ;7

        ld      a,15            ;7
        add     a,l             ;4
        ld      l,a             ;4
        jp      nc,bdone5        ;10
        inc     h               ;4
bdone5:  djnz    broutine5        ;13
        ret
