;##################################################################
;
;   TesbWan for TI-86 (Main program)
;
;   Programmed by Patrick Davidson (pad@calc.org)
;        
;   This program is in the public domain.  There is no warranty.
;
;   This file was last updated August 24, 2002.
;
;##################################################################

main_code:
        ld      hl,$f000
        ld      (hl),0
        ld      de,$f001
        ld      bc,$fff
        ldir

        xor     a
        ld      (exit_status),a
        call    init                    ; set up grayscale
        ld      a,(saved_flag)
        or      a
        jr      z,new_game

        ei
        xor     a
        ld      (saved_flag),a
        ld      hl,map_save
        call    unpack_level
        call    init_panel              ; set up panel
        jr      start_round

;############## Initialize new game

new_game:
        call    title_screen
        ei
        call    init_panel              ; set up panel

        ld      hl,0
        ld      (level),hl
        ld      h,$0005
        ld      (lives),hl
        ld      hl,level_data
        ld      (level_pointer),hl

;############## Initialize new level

next_level:
        ld      a,(level+1)
        cp      NUM_LEVELS
        jr      nz,no_win
        ld      a,1
        ld      (exit_status),a
        ld      b,10
loop_add:
        push    bc
        ld      hl,score+1
        ld      de,lives+1
        call    ADD_BCD
        pop     bc
        djnz    loop_add
        ret
no_win: call    initialize_level

;############## Initialize new round

next_round:
        xor     a
        ld      (screen_x),a
        ld      a,50
        ld      (paddle_x),a

        ld      hl,50<<8
        ld      (ball_x),hl
        ld      (ball_y),hl
        ld      a,4
        ld      (ball_angle),a
        xor     a
        ld      (ball_xdir),a
        ld      (ball_ydir),a
        ld      (ball_type),a
        ld      (long_paddle),a
        ld      (bonus_type),a
        ld      a,(game_speed)
        ld      (ball_speed),a
        ld      hl,0
        ld      (game_timer),hl
        ld      a,1
        ld      (paused),a

;############## Prepare screen

start_round:
        call    draw_level
        call    wait_next_frame
        call    show_level
        call    show_lives
        call    wait_next_frame
        call    show_level
        call    show_lives

        ld      hl,(ball_x)
        ld      a,(ball_y+1)
        ld      l,a
        ld      (prev_coords),hl

;############## Game main loop

main_loop:
        ld      a,(ball_type)           ; die if ball type = gone
        cp      2
        jp      z,died
                                        
        ld      a,(bricks_left)         ; next level if 0 bricks left
        or      a
        jp      z,next_level

        ld      hl,(game_timer)
        inc     hl
        ld      (game_timer),hl
        ld      a,7                     ; speed up every minute
        cp      h
        jr      nz,nospeedup
        ld      hl,ball_speed
        inc     (hl)
        xor     a
        ld      (game_timer+1),a
nospeedup:

        call    show_panel              ; display int count / score
        call    wait_next_frame         ; wait for next VBL, swap buffers
        call    show_background         ; display world

        ld      hl,lives_changed        ; redraw lives if needed
        ld      a,(hl)
        or      a
        jr      z,no_lives_changed
        ld      (hl),0
        call    show_lives
no_lives_changed:

        call    CONTRAST_GET_KEY        ; check control keys
        cp      K_ENTER
        jr      nz,no_pause
        ld      (paused),a
no_pause:
        cp      K_MORE
        jp      z,save_game
        cp      K_EXIT
        ret     z

        call    handle_paddle           ; move, draw paddle
        call    handle_ball             ; move, bounce, draw ball
        call    handle_bonus            ; move, collect, draw bonus

        ld      hl,paused
        bit     0,(hl)
        jr      z,main_loop

;############## Pause game

        ld      (hl),0
        call    show_panel

        ld      hl,(gfx_buffer)
        ld      a,h
        xor     8
        ld      d,a
        ld      e,l
        ld      bc,$800
        ldir

        ld      a,(gfx_buffer+1)
        cp      $F8
        call    nz,wait_next_frame

        ld      hl,$360a
        ld      a,(ball_y+1)
        cp      $36
        jr      c,bypok
        ld      hl,$300a
bypok:  ld      (_penCol),hl
        ld      hl,pause_msg
        call    _vputs

        ld      de,$fb00
lcpt:   ld      hl,$ff00
        ld      bc,$c0
        ld      e,$00
        ldir
        ld      a,d
        sub     4
        ld      d,a
        cp      $ef
        jr      nz,lcpt

pause_loop:
        call    wait_next_frame
        call    CONTRAST_GET_KEY
        cp      K_SECOND
        jr      nz,pause_loop
        jp      main_loop

pause_msg:
        .db     "Paused: Press 2nd to continue",0

;############## Player died

died:   xor     a
        ld      a,(lives+1)
        dec     a
        daa
        ld      (lives+1),a
        jp      nz,next_round
        inc     a
        ld      (exit_status),a
        ret

;############## Save game

save_game:                              ; save the game          
        ld      a,1
        ld      (saved_flag),a

pack_data:                              ; store working map in packed format
        ld      c,55
        ld      hl,working_map
        ld      de,map_save
loop_pack_map:
        ld      b,4
        xor     a
loop_byte:
        add     a,a
        add     a,a
        or      (hl)
        inc     hl
        djnz    loop_byte
        ld      (de),a
        inc     de
        dec     c
        jr      nz,loop_pack_map
        ret
